import {FoodProduct} from "./foodProduct.entity";
import {getTestEmissionFactor} from "../seed-dev-data";
import {FoodProductIngredient} from "./foodProductIngredient.entity";

let hamCheesePizza: FoodProduct;

beforeEach(async () => {
	hamCheesePizza = new FoodProduct({
		name: "hamCheesePizza",
		ingredients: [
			{ name: "ham", quantity: 0.1, unit: "kg" },
			{ name: "cheese", quantity: 0.15, unit: "kg" },
			{ name: "tomato", quantity: 0.4, unit: "kg" },
			{ name: "flour", quantity: 0.7, unit: "kg" },
			{ name: "oliveOil", quantity: 0.3, unit: "kg" },
		],
	});
});

describe("FoodProduct", () => {
	describe("constructor", () => {
		it("should create a food product with its ingredients", () => {
			expect(hamCheesePizza.name).toBe("hamCheesePizza");
			expect(hamCheesePizza.ingredients).toHaveLength(5);
			expect(hamCheesePizza.ingredients?.[1]?.name).toBe("cheese");
			expect(hamCheesePizza.ingredients?.[2]?.quantity).toBe(0.4);
			expect(hamCheesePizza.ingredients?.[3]?.unit).toBe("kg");
		});
	});

	describe("compute footprint", () => {
		it("shouldn't be able to compute the carbon footprint as carbon emission factors are unknown", () => {
			expect(hamCheesePizza.computeFootprint()).toBeNull();
			expect(hamCheesePizza.footprint).toBeNull();
		});

		it("should compute the carbon footprint for one ingredient but not the others", () => {
			// Set one ingredient test emission factor.
			(hamCheesePizza.ingredients as FoodProductIngredient[])[0].carbonEmissionFactor = getTestEmissionFactor("ham");
			// Compute its footprint.
			expect((hamCheesePizza.ingredients as FoodProductIngredient[])[0].computeFootprint()).toBeCloseTo(0.011, 3);
			expect(hamCheesePizza.computeFootprint()).toBeNull();
		});

		it("should compute the product carbon footprint", () => {
			// Set ingredients test emission factors.
			for (const ingredient of (hamCheesePizza.ingredients as FoodProductIngredient[]))
				ingredient.carbonEmissionFactor = getTestEmissionFactor(ingredient.name ?? "");

			// Compute product footprint.
			expect(hamCheesePizza.computeFootprint()).toBeCloseTo(0.224, 3);
			expect(hamCheesePizza.footprint).toBeCloseTo(0.224, 3);
		});
	});
});
