import {GreenlyDataSource, dataSource} from "../../config/dataSource";
import {seedTestCarbonEmissionFactors} from "../seed-dev-data";
import {FoodProductService} from "../foodProduct/foodProduct.service";
import {CarbonEmissionFactor} from "../carbonEmissionFactor/carbonEmissionFactor.entity";
import {FoodProductFootprintService} from "./foodProductFootprint.service";
import {FoodProduct} from "../foodProduct/foodProduct.entity";

let foodProductService: FoodProductService;
let foodProductFootprintService: FoodProductFootprintService;

beforeAll(async () => {
  await dataSource.initialize();
	foodProductService = new FoodProductService(
		dataSource.getRepository(FoodProduct),
		dataSource.getRepository(CarbonEmissionFactor),
	);
	foodProductFootprintService = new FoodProductFootprintService();
});

beforeEach(async () => {
  await GreenlyDataSource.cleanDatabase();
  await seedTestCarbonEmissionFactors();
});

describe("FoodProductFootprint.service", () => {
	it("should compute and save carbon footprint", async () => {
		let hamCheesePizza = await foodProductService.initialize({
			name: "hamCheesePizza",
			ingredients: [
				{ name: "ham", quantity: 0.1, unit: "kg" },
				{ name: "cheese", quantity: 0.15, unit: "kg" },
				{ name: "tomato", quantity: 0.4, unit: "kg" },
				{ name: "flour", quantity: 0.7, unit: "kg" },
				{ name: "oliveOil", quantity: 0.3, unit: "kg" },
			],
		});

		await foodProductFootprintService.computeAndSave(hamCheesePizza);

		expect(hamCheesePizza.footprint).toBeCloseTo(0.224, 3);

		const retrievedHamCheesePizza = await dataSource
			.getRepository(FoodProduct)
			.findOne({ where: { name: "hamCheesePizza" } });

		expect(retrievedHamCheesePizza?.name).toBe("hamCheesePizza");
		expect(retrievedHamCheesePizza?.footprint).toBeCloseTo(0.224, 3);
	});

	it("should fail to compute and save NULL carbon footprint", async () => {
		let hamCheesePizza = await foodProductService.initialize({
			name: "hamCheesePizza",
			ingredients: [
				{ name: "ham", quantity: 0.1, unit: "kg" },
				{ name: "cheese", quantity: 0.15, unit: "kg" },
				{ name: "tomato", quantity: 0.4, unit: "kg" },
				{ name: "flour", quantity: 0.7, unit: "kg" },
				{ name: "anyOil", quantity: 0.3, unit: "kg" },
			],
		});

		await foodProductFootprintService.computeAndSave(hamCheesePizza);

		expect(hamCheesePizza.footprint).toBeNull();

		const retrievedHamCheesePizza = await dataSource
			.getRepository(FoodProduct)
			.findOne({ where: { name: "hamCheesePizza" } });

		expect(retrievedHamCheesePizza?.name).toBe("hamCheesePizza");
		expect(retrievedHamCheesePizza?.footprint).toBeNull();
	});
});

afterAll(async () => {
	await dataSource.destroy();
});
